//
// dist.Transport - routines for marshalling and unmarshalling data over a
//                  socket connection

package dist;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class Transport
{
    //
    // Transport public constants

    public final static byte NILV = 0;
    public final static byte INTV = 1;
    public final static byte INTA = 2;
    public final static byte STRV = 3;
    public final static byte STRA = 4;

    //
    // Transport public static member functions

    public static void writeValue (DataOutputStream out, Object value)
         throws IOException
    {
        if (value == null) {
            out.writeByte(NILV);

        } else if (value instanceof Integer) {
            out.writeByte(INTV);
            out.writeInt(((Integer)value).intValue());

        } else if (value instanceof String) {
            out.writeByte(STRV);
            out.writeUTF((String)value);

        } else if (value instanceof int[]) {
            out.writeByte(INTA);
            int[] ints = (int[])value;
            out.writeInt(ints.length);
            for (int i = 0; i < ints.length; i++) out.writeInt(ints[i]);

        } else if (value instanceof String[]) {
            out.writeByte(STRA);
            String[] strs = (String[])value;
            out.writeInt(strs.length);
            for (int i = 0; i < strs.length; i++) out.writeUTF(strs[i]);

        } else {
            throw new IOException("Invalid type: " + value.getClass());
        }
    }

    public static Object readValue (DataInputStream in) throws IOException
    {
        byte type = in.readByte();

        switch (type) {
        case NILV:
            return null;

        case INTV:
            return new Integer(in.readInt());

        case INTA: {
            int[] ints = new int[in.readInt()];
            for (int i = 0; i < ints.length; i++) ints[i] = in.readInt();
            return ints;
        }

        case STRV:
            return in.readUTF();

        case STRA: {
            String[] strs = new String[in.readInt()];
            for (int i = 0; i < strs.length; i++) strs[i] = in.readUTF();
            return strs;
        }

        default:
            throw new IOException("Unknown type: " + type);
        }
    }
}
